package main

import (
	"dwelling-files/internal/http"
	"dwelling-files/web"
	"flag"
	"fmt"
	"log"
	"net/netip"
	"os"
	"os/signal"
	"strings"
	"syscall"
)

var version string

var listenAddress *string = flag.String("listen", "/var/run/dwelling-files/sock", "listen address (ip:port|unix_path)")
var directoryPath *string = flag.String("path", "/srv/ftp", "path to file share")
var enableFileHandler *bool = flag.Bool("file-handling", false, "enable file handling if it is handled by something else (e.g. NGiNX)")
var showVersion *bool = flag.Bool("v", false, "show version")

func main() {
	flag.Parse()
	log.SetFlags(0)

	if *showVersion {
		fmt.Println("dwelling-files ver.", version, "\nCopyright (c) 2022 Alexander \"Arav\" Andreev <me@arav.top>")
		return
	}

	var network string
	if !strings.ContainsRune(*listenAddress, ':') {
		network = "unix"
		defer os.Remove(*listenAddress)
	} else {
		ap, err := netip.ParseAddrPort(*listenAddress)
		if err != nil {
			log.Fatalln(err)
		}

		if ap.Addr().Is4() {
			network = "tcp4"
		} else if ap.Addr().Is6() {
			network = "tcp6"
		}
	}

	hand := http.New(directoryPath, web.Assets(), !*enableFileHandler)
	srv := http.NewHttpServer()

	srv.GET("/*filepath", hand.Index)

	if err := srv.Start(network, *listenAddress); err != nil {
		log.Fatalln(err)
	}

	doneSignal := make(chan os.Signal, 1)
	signal.Notify(doneSignal, os.Interrupt, syscall.SIGINT, syscall.SIGTERM)

	<-doneSignal

	if err := srv.Stop(); err != nil {
		log.Fatalln(err)
	}
}
